#!/bin/bash
#
# Compile the Less files of Bootstrap base.
#
# Deprecated script, please refer to the CSS command.

echo "This script is deprecated, please use:"
echo "  mdk css --compile"
echo ""

mdk css --compile
